/*******************************************************************************
*  WebHandler.cpp - Serve up the web content
*
*  berzerk/
*
*  Part of the BuckoSoft Network Monitor
*  Copyright(c) 2011 - BuckoSoft, Corp.
*
*  http://git.buckosoft.com/gitweb/?p=Berzerk.git;a=history;f=berzerk/WebHandler.cpp
*
\*******************************************************************************/

#include "stdafx.h"
#include "../config.h"
#include <iostream>
#include <fstream>
#include <sstream>
#include <string>

#if 0
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>
#include <xercesc/util/OutOfMemoryException.hpp>
#endif

#include <boost/lexical_cast.hpp>
#include <boost/foreach.hpp>
#include <boost/algorithm/string.hpp>
#include <boost/regex.hpp>
using namespace boost;

#include "server/xml.h"
#include "server/urlencode.h"
#include "server/reply.hpp"
#include "server/mime_types.hpp"
#include "Berzerk.h"
#include "Filter.h"
#include "TestFilter.h"
#include "WebHandler.h"

using namespace http::server;

#if 0
using namespace xercesc;
#endif

//-----------------------------------------------------------------------------


namespace berzerk {

using namespace statistics;


static PCSTR	s_public_statistics	= "/public/statistics";
static PCSTR	s_getTestSetup		= "/getTestSetup";
static PCSTR	s_testFilter		= "/testFilter?";
static PCSTR	s_status			= "/status";
static PCSTR	s_reject			= "/reject";
static PCSTR	s_localstats		= "/localstats";

int berzerk::WebHandler::DEBUG = 0;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
reply::status_type berzerk::WebHandler::handleRequest(const string& uri, const http::server::request& req, http::server::reply& rep) {
	if (uri == "/public")
		return(doPublicIndex(rep));

	string path = uri;
	size_t last_q_pos = uri.find_last_of("?");
	if (last_q_pos != string::npos)
		path = uri.substr(0, last_q_pos);
	// If the uri has a file extension, treat it like a file.
	size_t last_slash_pos = path.find_last_of("/");
	if (path[path.size() -1] == '/')
		path += "index.html";
	size_t last_dot_pos = path.find_last_of(".");
	string extension;
	if (last_dot_pos != string::npos && last_dot_pos > last_slash_pos)
		extension = path.substr(last_dot_pos + 1);
	if (extension.length() > 0)
		return(doFile(path, rep));
	if (uri.compare(0, strlen(s_getTestSetup), s_getTestSetup) == 0)
		return(doGetTestSetup(rep));
	if (uri.compare(0, strlen(s_public_statistics), s_public_statistics) == 0)
		return(doStatistics(uri.substr(strlen(s_public_statistics)), rep));
	if (uri.compare(0, strlen(s_localstats), s_localstats) == 0)
		return(doLocalStats(req.uri.substr(strlen(s_localstats)), rep));
	if (uri.compare(0, strlen(s_testFilter), s_testFilter) == 0)
		return(doTestFilter(req.uri.substr(strlen(s_testFilter)), rep));
	if (uri.compare(0, strlen(s_reject), s_reject) == 0)
		return(doReject(req.uri.substr(strlen(s_reject)), rep));
	if (uri.compare(0, strlen(s_status), s_status) == 0)
		return(doStatus(rep));
	cout << "Unhandled url: " << path << endl;
	return(reply::not_implemented);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
reply::status_type		berzerk::WebHandler::doPublicIndex(reply& rep) {
	rep.content += "LinesRead: ";
	rep.content += boost::lexical_cast<string>(statistics->getLinesRead());
	rep.content += crlf;
	rep.content += "IPsKilled: ";
	rep.content += boost::lexical_cast<string>(statistics->getIPsKilled());
	//rep.content += crlf;
	rep.content_type = reply::plain;
	return(reply::ok);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
reply::status_type		berzerk::WebHandler::doFile(const string& filename, reply& rep) {
	// Open the file to send back.
	string full_path = Filter::getRootDirectory() + "/web" + filename;
	if (DEBUG > 2)
		cout << "Returning file: " << full_path;
	ifstream is(full_path.c_str(), ios::in | ios::binary);
	if (!is) {
		rep = reply::stock_reply(reply::not_found);
		if (DEBUG > 2)
			cout << endl;
		cout << "404 - " << full_path << endl;
		return(reply::not_found);
	}

	size_t last_dot_pos = filename.find_last_of(".");
	string extension;
	extension = filename.substr(last_dot_pos + 1);
	if (DEBUG > 2)
		cout << " (" << mime_types::extension_to_type(extension) << ")" << endl;

	// Fill out the reply to be sent to the client.
	rep.status = reply::ok;
	char buf[512];
	while (is.read(buf, sizeof(buf)).gcount() > 0)
		rep.content.append(buf, is.gcount());
	rep.headers.resize(3);
	rep.headers[0].name = "Content-Length";
	rep.headers[0].value = boost::lexical_cast<string>(rep.content.size());
	rep.headers[1].name = "Content-Type";
	rep.headers[1].value = mime_types::extension_to_type(extension);
	rep.headers[2].name = "X-BuckoSoft";
	rep.headers[2].value = "approved";
	return(reply::ok);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
reply::status_type		berzerk::WebHandler::doStatistics(string args, reply& rep) {
	int count = 10;
	int	i;
	if (args.length() > 0 && args[0] == '?') {
		count = atoi(args.substr(1).c_str());
	}
	rep.content_type = reply::xml;
	rep.content += s_xml_preamble;
	rep.content += crlf;
	rep.content += "<statistics><overview>";
	rep.content += toXml("linesRead", statistics->getLinesRead());
	rep.content += toXml("ipsKilled", statistics->getIPsKilled());
	rep.content += "</overview>";
	statistics->lock();
	list<StatLine*>::iterator it;
	for (it=statistics->statLines.begin(), i=0; it != statistics->statLines.end() && i < count; it++, i++) {
		StatLine* sl = *it;
		rep.content += sl->toXML();
	}
	statistics->unlock();
	rep.content += "</statistics>";
	return(reply::ok);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
reply::status_type		berzerk::WebHandler::doLocalStats(string args, reply& rep) {
	int count = 10;
	int	i;
	if (args.length() > 0 && args[0] == '?') {
		count = atoi(args.substr(1).c_str());
	}
	rep.content_type = reply::xml;
	rep.content += s_xml_preamble;
	rep.content += crlf;
	rep.content += "<statistics><overview>";
	rep.content += toXml("linesRead", statistics->getLinesRead());
	rep.content += toXml("ipsKilled", statistics->getIPsKilled());
	rep.content += "</overview>";
	statistics->lock();
	list<StatLine*>::iterator it;
	for (it=statistics->statLines.begin(), i=0; it != statistics->statLines.end() && i < count; it++, i++) {
		StatLine* sl = *it;
		rep.content += sl->toXML();
	}
	statistics->unlock();
	rep.content += "</statistics>";
	return(reply::ok);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
reply::status_type		berzerk::WebHandler::doStatus(reply& rep) {
	list<LogFile*> logFiles = berzerk.getLogFiles();
	rep.content_type = reply::xml;
	rep.content += s_xml_preamble ;
	rep.content += "<logFiles>";
	rep.content += crlf;
	BOOST_FOREACH(LogFile* logFile, logFiles) {
		rep.content += toXml("logFile", logFile->name, 1);
	}
	rep.content += "</logFiles>\r\n";
	return(reply::ok);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
reply::status_type		berzerk::WebHandler::doReject(string args, reply& rep) {
	int	i;
	rep.content_type = reply::xml;
	rep.content += s_xml_preamble;
	rep.content += crlf;

	string rej;
	if (args.length() > 0 && args[0] == '?') {
		rej = urldecode(args.substr(1));
	} else {
		putXmlError(rep, "IP expected");
		return(reply::ok);
	}

	Filter filter("injectreject", "logkill", berzerk.getConfig().getThreshold());
	regex rex("");
	regex ipPattern("\\d\\.\\d\\.\\d\\.\\d");
	berzerk.lineMatched(&filter, rej, rex, ipPattern);
	rep.content += "<reject>";
	rep.content += "</reject>";
	return(reply::ok);
}

static PCSTR	s_name		= "name";
static PCSTR	s_pattern	= "pattern";
static PCSTR	s_testSetup	= "TestSetup";
static PCSTR	s_logFile	= "logFile";
static PCSTR	s_logFiles	= "logFiles";
static PCSTR	s_regexps	= "regexps";
static PCSTR	s_regexp	= "regexp";

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
reply::status_type		berzerk::WebHandler::doGetTestSetup(reply& rep) {
	list<LogFile*> logFiles = berzerk.getLogFiles();
	rep.content_type = reply::xml;
	rep.content += s_xml_preamble ;
	rep.content += xmlToken(s_testSetup, 0);
	rep.content += xmlToken(s_logFiles, 1);
	rep.content += crlf;
	BOOST_FOREACH(LogFile* logFile, logFiles) {
		rep.content += toXml("logFile", logFile->name, 2);
	}
	rep.content += xmlTokenE(s_logFiles, 1);
	rep.content += xmlToken(s_regexps, 1);
	BOOST_FOREACH(Filter* filter, berzerk.getFilters()) {
		rep.content += xmlToken(s_regexp, 2);
		rep.content += toXml(s_name, filter->getName(), 3);
		rep.content += toXml(s_logFile, filter->getLogFile(), 3);
		BOOST_FOREACH(string pat, filter->getMatchPatterns()) {
			rep.content += toXml(s_pattern, pat, 3);
		}
		rep.content += xmlTokenE(s_regexp, 2);
	}
	rep.content += xmlTokenE(s_regexps, 1);
	rep.content += xmlTokenE(s_testSetup, 0);
	return(reply::ok);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 * url params to testFilter:
 * %d index to log file
 * %d number of results to return
 * %s regex to match
 * %s regex to fetch ip
 */
reply::status_type		berzerk::WebHandler::doTestFilter(string args, reply& rep) {
	if (DEBUG > 2)
		cout << "args=" << args << endl;
	typedef vector< string > split_vector_type;
	split_vector_type sv;
	split(sv, args, is_any_of("|"));

	size_t sepa = args.find('|');
	if (sepa  != 3 || sv.size() != 4) {
		if (sepa <= 0) {
			rep.content = "Malformed request\n";
			rep.content += "Expected 4 parameters, got " + sv.size();
			return(reply::bad_request);
		}
	}
	//string df = args.substr(0, sepa);
	//string ftext = args.substr(sepa+1);
	string df = sv[0];
	string ml = sv[1];
	string ftext = urldecode(sv[2]);
	string itext = urldecode(sv[3]);
	const LogFile* logFile = berzerk.getLogFile(atoi(df.c_str()));
	if (!logFile)
		return(reply::bad_request);
	TestFilter* tf = new TestFilter(logFile, ftext, itext);
	tf->setMaxLines(atoi(ml.c_str()));
	tf->run();
	tf->output(rep);
	if (DEBUG > 2)
		cout << "df ='" << df << "' ftext = '" << ftext << "' itext = '" << itext << "'" << endl;
	delete tf;
	return(reply::ok);
}

}	// namespace berzerk
